class Position:
    def __init__(self, instrument_code, value, quantity, total_return):
        self.instrument_code = instrument_code
        self.value = value
        self.quantity = quantity
        self.total_return = total_return
